package flat.example;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.nio.ByteBuffer;

import org.junit.Test;

import com.google.flatbuffers.FlatBufferBuilder;

import flat.example.Action;
import flat.example.Attack;
import flat.example.Command;
import flat.example.Location;
import flat.example.Module;
import flat.example.Place;
import flat.example.Portal;
import flat.example.Weapon;

public class FlatBufferTest {

	@Test
	public void attack() throws Exception {
		FlatBufferBuilder fbb = new FlatBufferBuilder(10);
		int weapon = makeWeapon(fbb);
		Attack.startAttack(fbb);
		Attack.addWeapon(fbb, weapon);

		ByteBuffer bb = makeAction(fbb, Command.Attack, Attack.endAttack(fbb));

		Action attack = Action.getRootAsAction(bb);

		assertEquals(33333L, attack.userId());
		Location location = attack.from();
		assertEquals(3.333, location.latitude(), 0);
		assertEquals(2.8971, location.longitude(), 0);

		Portal target = attack.target();
		assertEquals(0x11, target.id(0));
		assertEquals(0x22, target.id(1));

		assertEquals(Command.Attack, attack.cmdType());
		Attack cmd = new Attack();
		assertNotNull(attack.cmd(cmd));
		assertEquals("ad2fe07d", cmd.weapon().hash());
	}

	ByteBuffer makeAction(FlatBufferBuilder fbb, byte type, int cmd) {
		int target = makePortal(fbb);

		Action.startAction(fbb);
		Action.addUserId(fbb, 33333L);
		Action.addFrom(fbb, Location.createLocation(fbb, 3.333, 2.8971));
		Action.addTarget(fbb, target);
		Action.addCmdType(fbb, type);
		Action.addCmd(fbb, cmd);
		int action = Action.endAction(fbb);

		Action.finishActionBuffer(fbb, action);
		return fbb.dataBuffer();
	}

	int makeWeapon(FlatBufferBuilder fbb) {
		int hash = fbb.createString("ad2fe07d");
		Weapon.startWeapon(fbb);
		Weapon.addHash(fbb, hash);
		return Weapon.endWeapon(fbb);
	}

	int makePortal(FlatBufferBuilder fbb) {
		int id = Portal.createIdVector(fbb, new byte[] { 0x11, 0x22 });
		Portal.startPortal(fbb);
		Portal.addId(fbb, id);
		return Portal.endPortal(fbb);
	}

	@Test
	public void place() throws Exception {
		FlatBufferBuilder fbb = new FlatBufferBuilder(10);
		int modules = Place.createModulesVector(fbb, makeModules(fbb));

		Place.startPlace(fbb);
		Place.addModules(fbb, modules);
		ByteBuffer bb = makeAction(fbb, Command.Place, Place.endPlace(fbb));

		Action place = Action.getRootAsAction(bb);

		assertEquals(Command.Place, place.cmdType());
		Place cmd = new Place();
		assertNotNull(place.cmd(cmd));

		Module firstMod = cmd.modules(0);
		assertEquals("firstModule", firstMod.hash());
		Module secondMod = cmd.modules(1);
		assertEquals("sencondModule", secondMod.hash());
	}

	int[] makeModules(FlatBufferBuilder fbb) {
		int firstHash = fbb.createString("firstModule");
		Module.startModule(fbb);
		Module.addHash(fbb, firstHash);
		int first = Module.endModule(fbb);

		int secondhash = fbb.createString("sencondModule");
		Module.startModule(fbb);
		Module.addHash(fbb, secondhash);
		int second = Module.endModule(fbb);

		return new int[] { first, second };
	}

}
