package msgpack;

import static org.junit.Assert.assertEquals;

import java.nio.file.Files;
import java.nio.file.Paths;

import org.junit.Test;
import org.msgpack.MessagePack;

public class MessagePackTest {

	@Test
	public void write() throws Exception {
		Action a = new Action();
		a.setUser_id(33333L);
		a.setDups("hogehoge");

		Location loc = new Location();
		loc.setLatitude(3.333);
		loc.setLongitude(2.8971);
		a.setFrom(loc);

		Portal portal = new Portal();
		portal.setId(new byte[] { 0x11, 0x22 });
		a.setTarget(portal);

		MessagePack mp = new MessagePack();
		mp.register(Location.class);
		mp.register(Portal.class);
		mp.register(Action.class);
		byte[] bytes = mp.write(a);
		Files.write(Paths.get("action.msg"), bytes);
	}

	@Test
	public void read() throws Exception {
		byte[] bytes = Files.readAllBytes(Paths.get("action.msg"));

		MessagePack mp = new MessagePack();
		mp.register(Location.class);
		mp.register(Portal.class);
		mp.register(Action.class);
		Action a = mp.read(bytes, Action.class);
		assertEquals(a.getUser_id(), 33333L);

		Location loc = a.getFrom();
		assertEquals(loc.getLatitude(), 3.333, 3);
		assertEquals(loc.getLongitude(), 2.8971, 0);

		Portal portal = a.getTarget();
		assertEquals(0x11, portal.getId()[0]);
		assertEquals(0x22, portal.getId()[1]);
	}
}
