/*
 * Copyright 2015 SATO taichi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package proto;

import static org.junit.Assert.assertEquals;

import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;

import org.junit.Test;

import proto.example.Example.Action;
import proto.example.Example.Location;
import proto.example.Example.Portal;

import com.google.protobuf.ByteString;
import com.google.protobuf.CodedOutputStream;

/**
 * @author taichi
 */
public class ProtocolBufferTest {

	@Test
	public void write() throws Exception {
		Location loc = Location.newBuilder().setLatitude(3.333)
				.setLongitude(2.8971).build();

		Portal por = Portal.newBuilder()
				.setId(ByteString.copyFrom(new byte[] { 0x11, 0x22 })).build();

		Action a = Action.newBuilder().setUserId(33333L).setDups("hogehoge")
				.setFrom(loc).setTarget(por).build();

		try (OutputStream os = Files.newOutputStream(Paths
				.get("action.proto.bin"))) {
			CodedOutputStream cos = CodedOutputStream.newInstance(os);
			a.writeTo(cos);
			cos.flush();
		}
	}

	@Test
	public void read() throws Exception {
		try (InputStream is = Files.newInputStream(Paths
				.get("action.proto.bin"))) {
			Action a = Action.parseFrom(is);
			assertEquals(a.getUserId(), 33333L);
			assertEquals(a.getDups(), "hogehoge");

			Location loc = a.getFrom();
			assertEquals(loc.getLatitude(), 3.333, 3);
			assertEquals(loc.getLongitude(), 2.8971, 0);

			// Portal portal = a.getTarget();
			// ByteString id = portal.getId();
			// assertEquals(0x11, id.byteAt(0));
			// assertEquals(0x22, id.byteAt(1));
		}
	}
}
